@@ -40,7 +40,16 @@ extern "C" {
 #include <assert.h>
 #include <time.h>
 #include <pthread.h>
+#if defined(HAVE_CPU_AFFINITY)
+#ifdef __FreeBSD__
+#include <sys/types.h>
+#include <sys/cpuset.h>
+#include <pthread_np.h>
+typedef cpuset_t cpu_set_t;
+#else
 #include <sched.h>
+#endif
+#endif
 
 #include <arpa/inet.h>
 #include <netinet/in.h>
@@ -1132,7 +1141,7 @@ long swoole_file_get_size(FILE *fp);
 int swoole_tmpfile(char *filename);
 swString* swoole_file_get_contents(char *filename);
 int swoole_file_put_contents(char *filename, char *content, size_t length);
-size_t swoole_file_size(char *filename);
+long swoole_file_size(char *filename);
 void swoole_open_remote_debug(void);
 char *swoole_dec2hex(int value, int base);
 int swoole_version_compare(char *version1, char *version2);
@@ -1868,8 +1877,6 @@ typedef struct
 
     swLock lock;
     swString *module_stack;
-    int call_php_func_argc;
-    int (*call_php_func)(const char *name);
     swHashMap *functions;
 
 } swServerG;
