@@ -44,7 +44,7 @@
 #include "Client.h"
 #include "async.h"
 
-#define PHP_SWOOLE_VERSION  "1.9.5"
+#define PHP_SWOOLE_VERSION  "1.9.6"
 #define PHP_SWOOLE_CHECK_CALLBACK
 
 /**
@@ -119,10 +119,6 @@ extern swoole_object_array swoole_objects;
 #ifndef HAVE_OPENSSL
 #error "Enable openssl support, require openssl library."
 #endif
-#else
-#ifdef SW_USE_HTTP2
-#error "Enable http2 support, require --enable-openssl."
-#endif
 #endif
 
 #ifdef SW_SOCKETS
@@ -138,9 +134,6 @@ extern swoole_object_array swoole_objects;
 #if !defined(HAVE_NGHTTP2)
 #error "Enable http2 support, require nghttp2 library."
 #endif
-#if !defined(HAVE_OPENSSL)
-#error "Enable http2 support, require openssl library."
-#endif
 #endif
 
 #include "php7_wrapper.h"
@@ -331,6 +324,12 @@ PHP_FUNCTION(swoole_timer_clear);
 PHP_FUNCTION(swoole_load_module);
 PHP_FUNCTION(swoole_strerror);
 PHP_FUNCTION(swoole_errno);
+//---------------------------------------------------------
+//                  serialize
+//---------------------------------------------------------
+PHP_FUNCTION(swoole_serialize);
+PHP_FUNCTION(swoole_fast_serialize);
+PHP_FUNCTION(swoole_unserialize);
 
 void swoole_destory_table(zend_resource *rsrc TSRMLS_DC);
 
@@ -353,6 +352,9 @@ void swoole_mysql_init(int module_number TSRMLS_DC);
 void swoole_module_init(int module_number TSRMLS_DC);
 void swoole_mmap_init(int module_number TSRMLS_DC);
 void swoole_channel_init(int module_number TSRMLS_DC);
+#if PHP_MAJOR_VERSION == 7
+void swoole_serialize_init(int module_number TSRMLS_DC);
+#endif
 
 int php_swoole_process_start(swWorker *process, zval *object TSRMLS_DC);
 
@@ -408,15 +410,21 @@ void php_swoole_get_recv_data(zval *zdata, swEventData *req, char *header, uint3
 int php_swoole_get_send_data(zval *zdata, char **str TSRMLS_DC);
 void php_swoole_onConnect(swServer *, swDataHead *);
 int php_swoole_onReceive(swServer *, swEventData *);
+int php_swoole_onPacket(swServer *, swEventData *);
 void php_swoole_onClose(swServer *, swDataHead *);
 void php_swoole_onBufferFull(swServer *, swDataHead *);
 void php_swoole_onBufferEmpty(swServer *, swDataHead *);
 int php_swoole_length_func(swProtocol *protocol, swConnection *conn, char *data, uint32_t length);
 
+#if PHP_MAJOR_VERSION >= 7
+PHPAPI zend_string* php_swoole_serialize(zval *zvalue);
+PHPAPI int php_swoole_unserialize(void *buffer, size_t len, zval *return_value, zval *object_args);
+#endif
+
 static sw_inline zval* php_swoole_server_get_callback(swServer *serv, int server_fd, int event_type)
 {
-    swListenPort *port = serv->connection_list[server_fd].object;
-    swoole_server_port_property *property = port->ptr;
+    swListenPort *port = (swListenPort *) serv->connection_list[server_fd].object;
+    swoole_server_port_property *property = (swoole_server_port_property *) port->ptr;
     if (!property)
     {
         return php_sw_server_callbacks[event_type];
@@ -444,7 +452,7 @@ ZEND_BEGIN_MODULE_GLOBALS(swoole)
     zend_bool display_errors;
     zend_bool cli;
     zend_bool use_namespace;
-    key_t message_queue_key;
+    zend_bool fast_serialize;
     uint32_t socket_buffer_size;
 ZEND_END_MODULE_GLOBALS(swoole)
 
