@@ -33,6 +33,7 @@ int swSocket_sendfile_sync(int sock, char *filename, off_t offset, double timeou
     if (fstat(file_fd, &file_stat) < 0)
     {
         swWarn("fstat() failed. Error: %s[%d]", strerror(errno), errno);
+        close(file_fd);
         return SW_ERR;
     }
 
@@ -143,18 +144,22 @@ int swSocket_wait_multi(int *list_of_fd, int n_fd, int timeout_ms, int events)
         int ret = poll(event_list, n_fd, timeout_ms);
         if (ret == 0)
         {
+            sw_free(event_list);
             return SW_ERR;
         }
         else if (ret < 0 && errno != EINTR)
         {
             swWarn("poll() failed. Error: %s[%d]", strerror(errno), errno);
+            sw_free(event_list);
             return SW_ERR;
         }
         else
         {
+            sw_free(event_list);
             return ret;
         }
     }
+    sw_free(event_list);
     return SW_OK;
 }
 
@@ -322,7 +327,7 @@ int swSocket_bind(int sock, int type, char *host, int *port)
         bzero(&addr_un, sizeof(addr_un));
         unlink(host);
         addr_un.sun_family = AF_UNIX;
-        strncpy(addr_un.sun_path, host, sizeof(addr_un.sun_path));
+        strncpy(addr_un.sun_path, host, sizeof(addr_un.sun_path) - 1);
         ret = bind(sock, (struct sockaddr*) &addr_un, sizeof(addr_un));
     }
     //IPv6
