@@ -103,6 +103,7 @@ int swClient_create(swClient *cli, int type, int async)
     if (!cli->socket)
     {
         swWarn("malloc(%d) failed.", (int ) sizeof(swConnection));
+        close(sockfd);
         return SW_ERR;
     }
 
@@ -487,7 +488,7 @@ static int swClient_tcp_pipe(swClient *cli, int write_fd, int flags)
     }
 
     int socktype;
-    socklen_t length;
+    socklen_t length = sizeof(socktype);
 
     if (flags & SW_CLIENT_PIPE_TCP_SESSION)
     {
