@@ -522,7 +522,12 @@ int swServer_worker_init(swServer *serv, swWorker *worker)
         {
             CPU_SET(SwooleWG.id % SW_CPU_NUM, &cpu_set);
         }
+#ifdef __FreeBSD__
+        if (cpuset_setaffinity(CPU_LEVEL_WHICH, CPU_WHICH_PID, -1,
+                                sizeof(cpu_set), &cpu_set) < 0)
+#else
         if (sched_setaffinity(getpid(), sizeof(cpu_set), &cpu_set) < 0)
+#endif
         {
             swSysError("sched_setaffinity() failed.");
         }
@@ -887,7 +892,7 @@ int swServer_udp_send(swServer *serv, swSendData *resp)
 int swServer_confirm(swServer *serv, int fd)
 {
     swConnection *conn = swServer_connection_verify(serv, fd);
-    if (!conn && !conn->listen_wait)
+    if (!conn || !conn->listen_wait)
     {
         return SW_ERR;
     }
@@ -1148,6 +1153,7 @@ swListenPort* swServer_add_port(swServer *serv, int type, char *host, int port)
     //bind address and port
     if (swSocket_bind(sock, ls->type, ls->host, &ls->port) < 0)
     {
+        close(sock);
         return NULL;
     }
     //dgram socket, setting socket buffer size
