@@ -23,8 +23,13 @@ static int openssl_init = 0;
 
 static const SSL_METHOD *swSSL_get_method(int method);
 static int swSSL_verify_callback(int ok, X509_STORE_CTX *x509_store);
-static RSA* swSSL_rsa512_key_callback(SSL *ssl, int is_export, int key_length);
-static int swSSL_set_dhparam(SSL_CTX* ssl_context);
+#ifndef OPENSSL_NO_RSA
+static RSA* swSSL_rsa_key_callback(SSL *ssl, int is_export, int key_length);
+#endif
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+static int swSSL_set_default_dhparam(SSL_CTX* ssl_context);
+#endif
+static int swSSL_set_dhparam(SSL_CTX* ssl_context, char *file);
 static int swSSL_set_ecdh_curve(SSL_CTX* ssl_context);
 
 #ifdef TLSEXT_TYPE_next_proto_neg
@@ -51,6 +56,10 @@ static const SSL_METHOD *swSSL_get_method(int method)
         return SSLv23_server_method();
     case SW_SSLv23_CLIENT_METHOD:
         return SSLv23_client_method();
+/**
+ * openssl 1.1.0
+ */
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     case SW_TLSv1_METHOD:
         return TLSv1_method();
     case SW_TLSv1_SERVER_METHOD:
@@ -79,6 +88,7 @@ static const SSL_METHOD *swSSL_get_method(int method)
         return DTLSv1_server_method();
     case SW_DTLSv1_CLIENT_METHOD:
         return DTLSv1_client_method();
+#endif
     case SW_SSLv23_METHOD:
     default:
         return SSLv23_method();
@@ -132,13 +142,22 @@ int swSSL_server_set_cipher(SSL_CTX* ssl_context, swSSL_config *cfg)
         }
     }
 
-#ifndef LIBRESSL_VERSION_NUMBER
-    SSL_CTX_set_tmp_rsa_callback(ssl_context, swSSL_rsa512_key_callback);
+#ifndef OPENSSL_NO_RSA
+    SSL_CTX_set_tmp_rsa_callback(ssl_context, swSSL_rsa_key_callback);
 #endif
 
-    if (strlen(cfg->ecdh_curve) > 0)
+    if (cfg->dhparam && strlen(cfg->dhparam) > 0)
+    {
+        swSSL_set_dhparam(ssl_context, cfg->dhparam);
+    }
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    else
+    {
+        swSSL_set_default_dhparam(ssl_context);
+    }
+#endif
+    if (cfg->ecdh_curve && strlen(cfg->ecdh_curve) > 0)
     {
-        swSSL_set_dhparam(ssl_context);
         swSSL_set_ecdh_curve(ssl_context);
     }
     return SW_OK;
@@ -341,6 +360,9 @@ int swSSL_get_client_certificate(SSL *ssl, char *buffer, size_t length)
 int swSSL_accept(swConnection *conn)
 {
     int n = SSL_do_handshake(conn->ssl);
+    /**
+     * The TLS/SSL handshake was successfully completed
+     */
     if (n == 1)
     {
         conn->ssl_state = SW_SSL_STATE_READY;
@@ -354,6 +376,14 @@ int swSSL_accept(swConnection *conn)
 #endif
         return SW_READY;
     }
+    /**
+     * The TLS/SSL handshake was not successful but was shutdown.
+     */
+    else if (n == 0)
+    {
+        return SW_ERROR;
+    }
+
     long err = SSL_get_error(conn->ssl, n);
     if (err == SSL_ERROR_WANT_READ)
     {
@@ -520,20 +550,38 @@ void swSSL_free_context(SSL_CTX* ssl_context)
     }
 }
 
-static RSA* swSSL_rsa512_key_callback(SSL *ssl, int is_export, int key_length)
+#ifndef OPENSSL_NO_RSA
+static RSA* swSSL_rsa_key_callback(SSL *ssl, int is_export, int key_length)
 {
-    static RSA *key;
-    if (key_length == 512)
+    static RSA *rsa_tmp = NULL;
+    if (rsa_tmp)
     {
-        if (key == NULL)
-        {
-            key = RSA_generate_key(512, RSA_F4, NULL, NULL);
-        }
+        return rsa_tmp;
     }
-    return key;
-}
 
-static int swSSL_set_dhparam(SSL_CTX* ssl_context)
+    BIGNUM *bn = BN_new();
+    if (bn == NULL)
+    {
+        swWarn("allocation error generating RSA key.");
+        return NULL;
+    }
+
+    if (!BN_set_word(bn, RSA_F4) || ((rsa_tmp = RSA_new()) == NULL)
+            || !RSA_generate_key_ex(rsa_tmp, key_length, bn, NULL))
+    {
+        if (rsa_tmp)
+        {
+            RSA_free(rsa_tmp);
+        }
+        rsa_tmp = NULL;
+    }
+    BN_free(bn);
+    return rsa_tmp;
+}
+#endif
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+static int swSSL_set_default_dhparam(SSL_CTX* ssl_context)
 {
     DH *dh;
     static unsigned char dh1024_p[] =
@@ -566,6 +614,7 @@ static int swSSL_set_dhparam(SSL_CTX* ssl_context)
     DH_free(dh);
     return SW_OK;
 }
+#endif
 
 static int swSSL_set_ecdh_curve(SSL_CTX* ssl_context)
 {
@@ -603,6 +652,34 @@ static int swSSL_set_ecdh_curve(SSL_CTX* ssl_context)
     return SW_OK;
 }
 
+static int swSSL_set_dhparam(SSL_CTX* ssl_context, char *file)
+{
+    DH *dh;
+    BIO *bio;
+
+    bio = BIO_new_file((char *) file, "r");
+    if (bio == NULL)
+    {
+        swWarn("BIO_new_file(%s) failed", file);
+        return SW_ERR;
+    }
+
+    dh = PEM_read_bio_DHparams(bio, NULL, NULL, NULL);
+    if (dh == NULL)
+    {
+        swWarn("PEM_read_bio_DHparams(%s) failed", file);
+        BIO_free(bio);
+        return SW_ERR;
+    }
+
+    SSL_CTX_set_tmp_dh(ssl_context, dh);
+
+    DH_free(dh);
+    BIO_free(bio);
+
+    return SW_OK;
+}
+
 #ifdef TLSEXT_TYPE_application_layer_protocol_negotiation
 
 static int swSSL_alpn_advertised(SSL *ssl, const uchar **out, uchar *outlen, const uchar *in, uint32_t inlen, void *arg)
