@@ -95,7 +95,6 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
 static int multipart_body_on_data(multipart_parser* p, const char *at, size_t length);
 static int multipart_body_on_header_complete(multipart_parser* p);
 static int multipart_body_on_data_end(multipart_parser* p);
-static int multipart_body_end(multipart_parser* p);
 
 static http_context* http_get_context(zval *object, int check_end TSRMLS_DC);
 
@@ -148,6 +147,7 @@ static PHP_METHOD(swoole_http_response, sendfile);
 static PHP_METHOD(swoole_http_response, cookie);
 static PHP_METHOD(swoole_http_response, rawcookie);
 static PHP_METHOD(swoole_http_response, header);
+static PHP_METHOD(swoole_http_response, initHeader);
 #ifdef SW_HAVE_ZLIB
 static PHP_METHOD(swoole_http_response, gzip);
 #endif
@@ -284,7 +284,7 @@ static const multipart_parser_settings mt_parser_settings =
     NULL,
     multipart_body_on_header_complete,
     multipart_body_on_data_end,
-    multipart_body_end
+    NULL,
 };
 
 const zend_function_entry swoole_http_server_methods[] =
@@ -303,6 +303,7 @@ const zend_function_entry swoole_http_request_methods[] =
 
 const zend_function_entry swoole_http_response_methods[] =
 {
+    PHP_ME(swoole_http_response, initHeader, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, cookie, arginfo_swoole_http_response_cookie, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, rawcookie, arginfo_swoole_http_response_cookie, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, status, arginfo_swoole_http_response_status, ZEND_ACC_PUBLIC)
@@ -404,7 +405,13 @@ static void http_parse_cookie(zval *array, const char *at, size_t length)
         else if (state == 1 && *_c == ';')
         {
             vlen = i - j;
-            strncpy(valbuf, (char * ) at + j, SW_HTTP_COOKIE_VALLEN);
+            if (vlen >= SW_HTTP_COOKIE_VALLEN)
+            {
+                swWarn("cookie value is too large.");
+                return;
+            }
+            memcpy(valbuf, (char *) at + j, vlen);
+            valbuf[vlen] = 0;
             vlen = php_url_decode(valbuf, vlen);
             if (klen > 1)
             {
@@ -431,8 +438,19 @@ static void http_parse_cookie(zval *array, const char *at, size_t length)
     if (j < length)
     {
         vlen = i - j;
+        if (klen >= SW_HTTP_COOKIE_KEYLEN)
+        {
+            swWarn("cookie key is too large.");
+            return;
+        }
         keybuf[klen - 1] = 0;
-        strncpy(valbuf, (char * ) at + j, SW_HTTP_COOKIE_VALLEN);
+        if (vlen >= SW_HTTP_COOKIE_VALLEN)
+        {
+            swWarn("cookie value is too large.");
+            return;
+        }
+        memcpy(valbuf, (char *) at + j, vlen);
+        valbuf[vlen] = 0;;
         vlen = php_url_decode(valbuf, vlen);
         if (klen > 1)
         {
@@ -556,11 +574,18 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
 #endif
 
     http_context *ctx = p->data;
-    zval *zrequest_object = ctx->request.zobject;
-    zval *zfiles = ctx->request.zfiles;
-    if (!zfiles)
+    /**
+     * Hash collision attack
+     */
+    if (ctx->input_var_num > PG(max_input_vars))
     {
-        swoole_http_server_array_init(files, request);
+        swoole_php_error(E_WARNING, "Input variables exceeded %ld. "
+                "To increase the limit change max_input_vars in php.ini.", PG(max_input_vars));
+        return SW_OK;
+    }
+    else
+    {
+        ctx->input_var_num++;
     }
 
     size_t header_len = ctx->current_header_name_len;
@@ -600,10 +625,9 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
         {
             ctx->current_input_name = estrndup(str, len);
 
-            zval *multipart_header;
-            SW_MAKE_STD_ZVAL(multipart_header);
+            zval *multipart_header = NULL;
+            SW_ALLOC_INIT_ZVAL(multipart_header);
             array_init(multipart_header);
-            add_assoc_zval(zfiles, ctx->current_input_name, multipart_header);
 
             sw_add_assoc_string(multipart_header, "name", "", 1);
             sw_add_assoc_string(multipart_header, "type", "", 1);
@@ -613,15 +637,15 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
 
             len = http_trim_double_quote(&filename, &str);
             sw_add_assoc_stringl(multipart_header, "name", str, len, 1);
+
+            ctx->current_multipart_header = multipart_header;
         }
         sw_zval_ptr_dtor(&tmp_array);
     }
 
-    if (strncasecmp(headername, "content-type", header_len) == 0)
+    if (strncasecmp(headername, "content-type", header_len) == 0 && ctx->current_multipart_header)
     {
-        zval *multipart_header = NULL;
-        sw_zend_hash_find(Z_ARRVAL_P(zfiles), ctx->current_input_name, strlen(ctx->current_input_name) + 1, (void **) &multipart_header);
-        sw_add_assoc_stringl(multipart_header, "type", (char * ) at, length, 1);
+        sw_add_assoc_stringl(ctx->current_multipart_header, "type", (char * ) at, length, 1);
     }
 
     if (ctx->current_header_name_allocated)
@@ -649,9 +673,7 @@ static int multipart_body_on_data(multipart_parser* p, const char *at, size_t le
     int n = fwrite(at, sizeof(char), length, (FILE *) p->fp);
     if (n != length)
     {
-        zval *files = ctx->request.zfiles;
-        zval *multipart_header = NULL;
-        sw_zend_hash_find(Z_ARRVAL_P(files), ctx->current_input_name, strlen(ctx->current_input_name) + 1, (void **) &multipart_header);
+        zval *multipart_header = ctx->current_multipart_header;
         add_assoc_long(multipart_header, "error", HTTP_UPLOAD_ERR_CANT_WRITE);
 
         fclose((FILE *) p->fp);
@@ -679,20 +701,18 @@ static void get_random_file_name(char *des, const char *src)
 
 static int multipart_body_on_header_complete(multipart_parser* p)
 {
+#if PHP_MAJOR_VERSION < 7
+    TSRMLS_FETCH_FROM_CTX(sw_thread_ctx ? sw_thread_ctx : NULL);
+#endif
+
     http_context *ctx = p->data;
     if (!ctx->current_input_name)
     {
         return 0;
     }
 
-    zval *files = ctx->request.zfiles;
-    zval *multipart_header;
-
-    if (sw_zend_hash_find(Z_ARRVAL_P(files), ctx->current_input_name, strlen(ctx->current_input_name) + 1, (void **) &multipart_header) == FAILURE)
-    {
-        return 0;
-    }
-
+    zval *multipart_header = ctx->current_multipart_header;
+    zval *zrequest_object = ctx->request.zobject;
     zval *zerr = NULL;
     sw_zend_hash_find(Z_ARRVAL_P(multipart_header), ZEND_STRS("error"), (void **) &zerr);
     if (Z_LVAL_P(zerr) != HTTP_UPLOAD_ERR_OK)
@@ -705,7 +725,7 @@ static int multipart_body_on_header_complete(multipart_parser* p)
     int tmpfile = swoole_tmpfile(file_path);
 
     FILE *fp = fdopen(tmpfile, "wb+");
-    if (fp < 0)
+    if (fp == NULL)
     {
         add_assoc_long(multipart_header, "error", HTTP_UPLOAD_ERR_NO_TMP_DIR);
         swWarn("fopen(%s) failed. Error %s[%d]", file_path, strerror(errno), errno);
@@ -715,6 +735,17 @@ static int multipart_body_on_header_complete(multipart_parser* p)
     p->fp = fp;
     sw_add_assoc_string(multipart_header, "tmp_name", file_path, 1);
 
+    zval *ztmpfiles = sw_zend_read_property(swoole_http_request_class_entry_ptr, zrequest_object, ZEND_STRL("tmpfiles"), 1 TSRMLS_CC);
+    if (ZVAL_IS_NULL(ztmpfiles))
+    {
+        swoole_http_server_array_init(tmpfiles, request);
+    }
+
+    int file_path_len = strlen(file_path);
+    sw_add_next_index_stringl(ztmpfiles, file_path, file_path_len, 1);
+    char *temp_filename = file_path;
+    sw_zend_hash_add(SG(rfc1867_uploaded_files), temp_filename, file_path_len + 1, &temp_filename, sizeof(char *), NULL);
+
     return 0;
 }
 
@@ -734,25 +765,8 @@ static int multipart_body_on_data_end(multipart_parser* p)
             swoole_http_server_array_init(post, request);
         }
 
-        char *name = ctx->current_form_data_name;
-        int len = ctx->current_form_data_name_len;
-
-        if ((name[len-1] == ']') && (name[len-2] == '['))
-        {
-            zval *array_value;
-            if (sw_zend_hash_find(Z_ARRVAL_P(zpost), name, len + 1, (void **) &array_value) == FAILURE)
-            {
-                SW_MAKE_STD_ZVAL(array_value);
-                array_init(array_value);
-                add_assoc_zval(zpost, name, array_value);
-            }
-            sw_add_next_index_stringl(array_value, swoole_http_form_data_buffer->str, swoole_http_form_data_buffer->length, 1);
-        }
-        else
-        {
-            sw_add_assoc_stringl_ex(zpost, ctx->current_form_data_name, ctx->current_form_data_name_len + 1,
-                    swoole_http_form_data_buffer->str, swoole_http_form_data_buffer->length, 1);
-        }
+        php_register_variable_safe(ctx->current_form_data_name, swoole_http_form_data_buffer->str,
+                swoole_http_form_data_buffer->length, zpost TSRMLS_CC);
 
         efree(ctx->current_form_data_name);
         ctx->current_form_data_name = NULL;
@@ -766,54 +780,32 @@ static int multipart_body_on_data_end(multipart_parser* p)
         return 0;
     }
 
-    zval *files = ctx->request.zfiles;
-    if (ZVAL_IS_NULL(files))
-    {
-        return 0;
-    }
-
-    zval *multipart_header = NULL;
-    sw_zend_hash_find(Z_ARRVAL_P(files), ctx->current_input_name, strlen(ctx->current_input_name) + 1, (void **) &multipart_header);
-
+    zval *multipart_header = ctx->current_multipart_header;
     if (p->fp != NULL)
     {
         long size = swoole_file_get_size((FILE*) p->fp);
         add_assoc_long(multipart_header, "size", size);
 
-        fclose((FILE *)p->fp);
+        fclose((FILE *) p->fp);
         p->fp = NULL;
     }
 
-    efree(ctx->current_input_name);
-
-    return 0;
-}
-
-static int multipart_body_end(multipart_parser* p)
-{
-#if PHP_MAJOR_VERSION < 7
-    TSRMLS_FETCH_FROM_CTX(sw_thread_ctx ? sw_thread_ctx : NULL);
-#endif
-
-    http_context *ctx = p->data;
-    zval *files = ctx->request.zfiles;
-    zval *value;
-
-    SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(files), value)
+    zval *zfiles = ctx->request.zfiles;
+    if (!zfiles)
     {
-        zval *file_path;
-        if (sw_zend_hash_find(Z_ARRVAL_P(value), ZEND_STRS("tmp_name"), (void **) &file_path) == SUCCESS)
-        {
-#if PHP_MAJOR_VERSION >= 7
-            zend_string *tmp_name = zval_get_string(file_path);
-            zend_hash_add_ptr(SG(rfc1867_uploaded_files), tmp_name, tmp_name);
-#else
-            char *temp_filename = Z_STRVAL_P(file_path);
-            sw_zend_hash_add(SG(rfc1867_uploaded_files), temp_filename, Z_STRLEN_P(file_path) + 1, &temp_filename, sizeof(char *), NULL);
-#endif
-        }
+        swoole_http_server_array_init(files, request);
     }
-    SW_HASHTABLE_FOREACH_END();
+
+    php_register_variable_ex(ctx->current_input_name, multipart_header, zfiles TSRMLS_CC);
+
+    efree(ctx->current_input_name);
+    ctx->current_input_name = NULL;
+#if PHP_MAJOR_VERSION >= 7
+    efree(ctx->current_multipart_header);
+#else
+    sw_zval_ptr_dtor(&multipart_header);
+#endif
+    ctx->current_multipart_header = NULL;
 
     return 0;
 }
@@ -1319,6 +1311,8 @@ static char *http_status_message(int code)
         return "502 Bad Gateway";
     case 503:
         return "503 Service Unavailable";
+    case 504:
+        return "504 Gateway Timeout";
     case 505:
         return "505 HTTP Version Not Supported";
     case 506:
@@ -1480,31 +1474,24 @@ static PHP_METHOD(swoole_http_request, rawcontent)
 
 static PHP_METHOD(swoole_http_request, __destruct)
 {
-    zval *zfiles = sw_zend_read_property(swoole_http_request_class_entry_ptr, getThis(), ZEND_STRL("files"), 1 TSRMLS_CC);
+    zval *ztmpfiles = sw_zend_read_property(swoole_http_request_class_entry_ptr, getThis(), ZEND_STRL("tmpfiles"), 1 TSRMLS_CC);
     //upload files
-    if (zfiles && Z_TYPE_P(zfiles) == IS_ARRAY)
+    if (ztmpfiles && Z_TYPE_P(ztmpfiles) == IS_ARRAY)
     {
-        zval *value;
-        char *key;
-        int keytype;
-        uint32_t keylen;
-
-        SW_HASHTABLE_FOREACH_START2(Z_ARRVAL_P(zfiles), key, keylen, keytype, value)
+        zval *file_path;
+        SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(ztmpfiles), file_path)
         {
-            if (HASH_KEY_IS_STRING != keytype)
+            if (Z_TYPE_P(file_path) != IS_STRING)
             {
                 continue;
             }
-            zval *file_path;
-            if (sw_zend_hash_find(Z_ARRVAL_P(value), ZEND_STRS("tmp_name"), (void **) &file_path) == SUCCESS)
+            unlink(Z_STRVAL_P(file_path));
+            if (SG(rfc1867_uploaded_files))
             {
-                unlink(Z_STRVAL_P(file_path));
                 sw_zend_hash_del(SG(rfc1867_uploaded_files), Z_STRVAL_P(file_path), Z_STRLEN_P(file_path) + 1);
             }
         }
         SW_HASHTABLE_FOREACH_END();
-        (void)key;
-        (void)keylen;
     }
     swoole_set_object(getThis(), NULL);
 }
@@ -1647,10 +1634,6 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
             {
                 flag |= HTTP_RESPONSE_CONNECTION;
             }
-            else if (strncasecmp(key, "Content-Length", keylen) == 0)
-            {
-                flag |= HTTP_RESPONSE_CONTENT_LENGTH;
-            }
             else if (strncasecmp(key, "Date", keylen) == 0)
             {
                 flag |= HTTP_RESPONSE_DATE;
@@ -1680,26 +1663,9 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
                 swString_append_ptr(response, ZEND_STRL("Connection: close\r\n"));
             }
         }
-        if (ctx->request.method == PHP_HTTP_OPTIONS)
+        if (!(flag & HTTP_RESPONSE_CONTENT_TYPE))
         {
-            swString_append_ptr(response, ZEND_STRL("Allow: GET, POST, PUT, PATCH, DELETE, HEAD, OPTIONS\r\nContent-Length: 0\r\n"));
-        }
-        else if (body_length > 0)
-        {
-
-#ifdef SW_HAVE_ZLIB
-            if (ctx->gzip_enable)
-            {
-                body_length = swoole_zlib_buffer->length;
-            }
-#endif
-            n = snprintf(buf, sizeof(buf), "Content-Length: %d\r\n", body_length);
-            swString_append_ptr(response, buf, n);
-
-            if (!(flag & HTTP_RESPONSE_CONTENT_TYPE))
-            {
-                swString_append_ptr(response, ZEND_STRL("Content-Type: text/html\r\n"));
-            }
+            swString_append_ptr(response, ZEND_STRL("Content-Type: text/html\r\n"));
         }
         if (!(flag & HTTP_RESPONSE_DATE))
         {
@@ -1711,6 +1677,7 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
     }
     else
     {
+        //Server
         swString_append_ptr(response, ZEND_STRL("Server: "SW_HTTP_SERVER_SOFTWARE"\r\nContent-Type: text/html\r\n"));
         if (ctx->keepalive)
         {
@@ -1720,34 +1687,33 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
         {
             swString_append_ptr(response, ZEND_STRL("Connection: close\r\n"));
         }
-
+        //Date
         date_str = sw_php_format_date(ZEND_STRL(SW_HTTP_DATE_FORMAT), SwooleGS->now, 0 TSRMLS_CC);
         n = snprintf(buf, sizeof(buf), "Date: %s\r\n", date_str);
         efree(date_str);
         swString_append_ptr(response, buf, n);
-
-        if (ctx->request.method == PHP_HTTP_OPTIONS)
-        {
-            n = snprintf(buf, sizeof(buf), "Allow: GET, POST, PUT, DELETE, HEAD, OPTIONS\r\nContent-Length: %d\r\n", 0);
-            swString_append_ptr(response, buf, n);
-        }
-        else if (body_length > 0)
-        {
-#ifdef SW_HAVE_ZLIB
-            if (ctx->gzip_enable)
-            {
-                body_length = swoole_zlib_buffer->length;
-            }
-#endif
-            n = snprintf(buf, sizeof(buf), "Content-Length: %d\r\n", body_length);
-            swString_append_ptr(response, buf, n);
-        }
     }
-
+    /**
+     * Http chunk
+     */
     if (ctx->chunk)
     {
         swString_append_ptr(response, SW_STRL("Transfer-Encoding: chunked\r\n") - 1);
     }
+    /**
+     * Content-Length
+     */
+    else
+    {
+#ifdef SW_HAVE_ZLIB
+        if (ctx->gzip_enable)
+        {
+            body_length = swoole_zlib_buffer->length;
+        }
+#endif
+        n = snprintf(buf, sizeof(buf), "Content-Length: %d\r\n", body_length);
+        swString_append_ptr(response, buf, n);
+    }
     //http cookies
     if (ctx->response.zcookie)
     {
@@ -1837,6 +1803,27 @@ static int http_response_compress(swString *body, int level)
 }
 #endif
 
+static PHP_METHOD(swoole_http_response, initHeader)
+{
+    http_context *ctx = http_get_context(getThis(), 0 TSRMLS_CC);
+    if (!ctx)
+    {
+        RETURN_FALSE;
+    }
+    zval *zresponse_object = ctx->response.zobject;
+    zval *zheader = ctx->response.zheader;
+    if (!zheader)
+    {
+        swoole_http_server_array_init(header, response);
+    }
+
+    zval *zcookie = ctx->response.zcookie;
+    if (!zcookie)
+    {
+        swoole_http_server_array_init(cookie, response);
+    }
+}
+
 static PHP_METHOD(swoole_http_response, end)
 {
     zval *zdata = NULL;
